@@ -88,7 +88,14 @@ def parse_args(args=None):
 
     parser.add_argument('--augment_nopunct', type=float, default=None, help='Augment the training data by copying this fraction of punct-ending sentences as non-punct.  Default of None will aim for roughly 10%')
 
+    parser.add_argument('--wandb', action='store_true', help='Start a wandb session and write the results of training.  Only applies to training.  Use --wandb_name instead to specify a name')
+    parser.add_argument('--wandb_name', default=None, help='Name of a wandb session to start when training.  Will default to the dataset short name')
+
     args = parser.parse_args(args=args)
+
+    if args.wandb_name:
+        args.wandb = True
+
     return args
 
 def main(args=None):
@@ -158,6 +165,13 @@ def train(args):
         logger.info("Skip training because no data available...")
         sys.exit(0)
 
+    if args['wandb']:
+        import wandb
+        wandb_name = args['wandb_name'] if args['wandb_name'] else "%s_depparse" % args['shorthand']
+        wandb.init(name=wandb_name, config=args)
+        wandb.run.define_metric('train_loss', summary='min')
+        wandb.run.define_metric('dev_score', summary='max')
+
     logger.info("Training parser...")
     trainer = Trainer(args=args, vocab=vocab, pretrain=pretrain, use_cuda=args['cuda'])
 
@@ -199,6 +213,10 @@ def train(args):
 
                 train_loss = train_loss / args['eval_interval'] # avg loss per batch
                 logger.info("step {}: train_loss = {:.6f}, dev_score = {:.4f}".format(global_step, train_loss, dev_score))
+
+                if args['wandb']:
+                    wandb.log({'train_loss': train_loss, 'dev_score': dev_score})
+
                 train_loss = 0
 
                 # save best model
@@ -230,8 +248,16 @@ def train(args):
 
     logger.info("Training ended with {} steps.".format(global_step))
 
-    best_f, best_eval = max(dev_score_history)*100, np.argmax(dev_score_history)+1
-    logger.info("Best dev F1 = {:.2f}, at iteration = {}".format(best_f, best_eval * args['eval_interval']))
+    if args['wandb']:
+        wandb.finish()
+
+    if len(dev_score_history) > 0:
+        best_f, best_eval = max(dev_score_history)*100, np.argmax(dev_score_history)+1
+        logger.info("Best dev F1 = {:.2f}, at iteration = {}".format(best_f, best_eval * args['eval_interval']))
+    else:
+        logger.info("Dev set never evaluated.  Saving final model.")
+        trainer.save(model_file)
+
 
 def evaluate(args):
     # file paths
