@@ -1,5 +1,8 @@
+"""
+This class runs a Java process to evaluate a treebank prediction using CoreNLP
+"""
 
-
+from collections import namedtuple
 
 import stanza
 from stanza.protobuf import EvaluateParserRequest, EvaluateParserResponse
@@ -8,6 +11,9 @@ from stanza.server.java_protobuf_requests import send_request, build_tree, JavaP
 
 EVALUATE_JAVA = "edu.stanford.nlp.parser.metrics.EvaluateExternalParser"
 
+ParseResult = namedtuple("ParseResult", ['gold', 'predictions'])
+ScoredTree = namedtuple("ScoredTree", ['tree', 'score'])
+
 def build_request(treebank):
     """
     treebank should be a list of pairs:  [gold, predictions]
@@ -19,7 +25,12 @@ def build_request(treebank):
     for gold, predictions in treebank:
         parse_result = request.treebank.add()
         parse_result.gold.CopyFrom(build_tree(gold, None))
-        for prediction, score in predictions:
+        for pred in predictions:
+            if isinstance(pred, tuple):
+                prediction, score = pred
+            else:
+                prediction = pred
+                score = None
             parse_result.predicted.append(build_tree(prediction, score))
 
     return request
@@ -32,8 +43,13 @@ class EvaluateParser(JavaProtobufContext):
     This is a context window which keeps a process open.  Should allow
     for multiple requests without launching new java processes each time.
     """
-    def __init__(self, classpath=None):
-        super(EvaluateParser, self).__init__(classpath, EvaluateParserResponse, EVALUATE_JAVA)
+    def __init__(self, classpath=None, kbest=None):
+        if kbest is not None:
+            extra_args = ["-evalPCFGkBest", "{}".format(kbest), "-evals", "pcfgTopK"]
+        else:
+            extra_args = []
+
+        super(EvaluateParser, self).__init__(classpath, EvaluateParserResponse, EVALUATE_JAVA, extra_args=extra_args)
 
     def process(self, treebank):
         request = build_request(treebank)
