@@ -26,14 +26,14 @@ import glob
 import os
 import random
 import re
-import shutil
-import subprocess
 import tempfile
 
 from collections import Counter
 
+from stanza.models.common.constant import treebank_to_short_name
 import stanza.utils.datasets.common as common
-import stanza.utils.datasets.prepare_tokenizer_data as prepare_tokenizer_data
+from stanza.utils.datasets.common import read_sentences_from_conllu, write_sentences_to_conllu, INT_RE, MWT_RE, MWT_OR_COPY_RE
+import stanza.utils.datasets.tokenization.convert_my_alt as convert_my_alt
 import stanza.utils.datasets.tokenization.convert_vi_vlsp as convert_vi_vlsp
 import stanza.utils.datasets.tokenization.convert_th_best as convert_th_best
 import stanza.utils.datasets.tokenization.convert_th_lst20 as convert_th_lst20
@@ -43,7 +43,10 @@ def copy_conllu_file(tokenizer_dir, tokenizer_file, dest_dir, dest_file, short_n
     original = f"{tokenizer_dir}/{short_name}.{tokenizer_file}.conllu"
     copied = f"{dest_dir}/{short_name}.{dest_file}.conllu"
 
-    shutil.copyfile(original, copied)
+    print("Copying from %s to %s" % (original, copied))
+    # do this instead of shutil.copyfile in case there are manipulations needed
+    sents = read_sentences_from_conllu(original)
+    write_sentences_to_conllu(copied, sents)
 
 def copy_conllu_treebank(treebank, paths, dest_dir, postprocess=None, augment=True):
     """
@@ -53,7 +56,7 @@ def copy_conllu_treebank(treebank, paths, dest_dir, postprocess=None, augment=Tr
     """
     os.makedirs(dest_dir, exist_ok=True)
 
-    short_name = common.project_to_short_name(treebank)
+    short_name = treebank_to_short_name(treebank)
     short_language = short_name.split("_")[0]
 
     with tempfile.TemporaryDirectory() as tokenizer_dir:
@@ -78,31 +81,7 @@ def copy_conllu_treebank(treebank, paths, dest_dir, postprocess=None, augment=Tr
         postprocess(tokenizer_dir, "test.gold", dest_dir, "test.gold", short_name)
         copy_conllu_file(dest_dir, "test.gold", dest_dir, "test.in", short_name)
 
-def read_sentences_from_conllu(filename):
-    sents = []
-    cache = []
-    with open(filename) as infile:
-        for line in infile:
-            line = line.strip()
-            if len(line) == 0:
-                if len(cache) > 0:
-                    sents += [cache]
-                    cache = []
-                continue
-            cache += [line]
-        if len(cache) > 0:
-            sents += [cache]
-    return sents
-
-def write_sentences_to_conllu(filename, sents):
-    with open(filename, 'w') as outfile:
-        for lines in sents:
-            for line in lines:
-                print(line, file=outfile)
-            print("", file=outfile)
-
-def split_train_file(treebank, train_input_conllu,
-                     train_output_conllu, dev_output_conllu):
+def split_train_file(treebank, train_input_conllu, train_output_conllu, dev_output_conllu):
     # set the seed for each data file so that the results are the same
     # regardless of how many treebanks are processed at once
     random.seed(1234)
@@ -117,8 +96,8 @@ def split_train_file(treebank, train_input_conllu,
     # split conllu data
     dev_sents = sents[:n_dev]
     train_sents = sents[n_dev:]
-    print("Train/dev split not present.  Randomly splitting train file")
-    print(f"{len(sents)} total sentences found: {n_train} in train, {n_dev} in dev.")
+    print("Train/dev split not present.  Randomly splitting train file from %s to %s and %s" % (train_input_conllu, train_output_conllu, dev_output_conllu))
+    print(f"{len(sents)} total sentences found: {n_train} in train, {n_dev} in dev")
 
     # write conllu
     write_sentences_to_conllu(train_output_conllu, train_sents)
@@ -126,49 +105,6 @@ def split_train_file(treebank, train_input_conllu,
 
     return True
 
-def mwt_name(base_dir, short_name, dataset):
-    return f"{base_dir}/{short_name}-ud-{dataset}-mwt.json"
-
-def prepare_dataset_labels(input_txt, input_conllu, tokenizer_dir, short_name, dataset):
-    prepare_tokenizer_data.main([input_txt,
-                                 input_conllu,
-                                 "-o", f"{tokenizer_dir}/{short_name}-ud-{dataset}.toklabels",
-                                 "-m", mwt_name(tokenizer_dir, short_name, dataset)])
-
-def prepare_treebank_labels(tokenizer_dir, short_name):
-    for dataset in ("train", "dev", "test"):
-        output_txt = f"{tokenizer_dir}/{short_name}.{dataset}.txt"
-        output_conllu = f"{tokenizer_dir}/{short_name}.{dataset}.gold.conllu"
-        try:
-            prepare_dataset_labels(output_txt, output_conllu, tokenizer_dir, short_name, dataset)
-        except (KeyboardInterrupt, SystemExit):
-            raise
-        except:
-            print("Failed to convert %s to %s" % (output_txt, output_conllu))
-            raise
-
-CONLLU_TO_TXT_PERL = os.path.join(os.path.split(__file__)[0], "conllu_to_text.pl")
-
-def convert_conllu_to_txt(tokenizer_dir, short_name, shards=("train", "dev", "test")):
-    for dataset in shards:
-        output_conllu = f"{tokenizer_dir}/{short_name}.{dataset}.gold.conllu"
-        output_txt = f"{tokenizer_dir}/{short_name}.{dataset}.txt"
-
-        if not os.path.exists(output_conllu):
-            # the perl script doesn't raise an error code for file not found!
-            raise FileNotFoundError("Cannot convert %s as the file cannot be found" % output_conllu)
-        # use an external script to produce the txt files
-        subprocess.check_output(f"perl {CONLLU_TO_TXT_PERL} {output_conllu} > {output_txt}", shell=True)
-
-
-# RE to see if the index of a conllu line represents an MWT
-MWT_RE = re.compile("^[0-9]+[-][0-9]+")
-
-# RE to see if the index of a conllu line represents an MWT or copy node
-MWT_OR_COPY_RE = re.compile("^[0-9]+[-.][0-9]+")
-
-# more restrictive than an actual int as we expect certain formats in the conllu files
-INT_RE = re.compile("^[0-9]+$")
 
 def strip_mwt_from_sentences(sents):
     """
@@ -625,6 +561,8 @@ def find_text_idx(sentence):
             return idx
     return -1
 
+DIGIT_RE = re.compile("[0-9]")
+
 def change_indices(line, delta):
     """
     Adjust all indices in the given sentence by delta.  Useful when removing a word, for example
@@ -640,17 +578,23 @@ def change_indices(line, delta):
         return line
 
     if MWT_OR_COPY_RE.match(pieces[0]):
-        raise NotImplementedError("Need to implement change_indices for copy nodes")
-
-    if not INT_RE.match(pieces[0]):
+        index_pieces = pieces[0].split(".", maxsplit=1)
+        pieces[0] = "%d.%s" % (int(index_pieces[0]) + delta, index_pieces[1])
+    elif not INT_RE.match(pieces[0]):
         raise NotImplementedError("Unknown index type: %s" % pieces[0])
-
-    pieces[0] = str(int(pieces[0]) + delta)
-    dep = int(pieces[6])
-    if dep != 0:
-        pieces[6] = str(int(dep) + delta)
+    else:
+        pieces[0] = str(int(pieces[0]) + delta)
+    if pieces[6] != '_':
+        # copy nodes don't have basic dependencies in the es_ancora treebank
+        dep = int(pieces[6])
+        if dep != 0:
+            pieces[6] = str(int(dep) + delta)
     if pieces[8] != '_':
-        raise NotImplementedError("Need to handle the additional deps field in change_indices")
+        dep_pieces = pieces[8].split(":", maxsplit=1)
+        if DIGIT_RE.search(dep_pieces[1]):
+            raise NotImplementedError("Need to handle multiple additional deps:\n%s" % line)
+        if int(dep_pieces[0]) != 0:
+            pieces[8] = str(int(dep_pieces[0]) + delta) + ":" + dep_pieces[1]
     line = "\t".join(pieces)
     return line
 
@@ -790,10 +734,11 @@ def build_combined_korean_dataset(udbase_dir, tokenizer_dir, short_name, dataset
 
 def build_combined_korean(udbase_dir, tokenizer_dir, short_name):
     for dataset in ("train", "dev", "test"):
-        output_conllu = f"{tokenizer_dir}/{short_name}.{dataset}.gold.conllu"
+        output_conllu = common.tokenizer_conllu_name(tokenizer_dir, short_name, dataset)
         build_combined_korean_dataset(udbase_dir, tokenizer_dir, short_name, dataset, output_conllu)
 
-def build_combined_italian_dataset(udbase_dir, tokenizer_dir, handparsed_dir, short_name, dataset):
+def build_combined_italian_dataset(paths, dataset):
+    udbase_dir = paths["UDBASE"]
     if dataset == 'train':
         # could maybe add ParTUT, but that dataset has a slightly different xpos set
         # (no DE or I)
@@ -806,15 +751,6 @@ def build_combined_italian_dataset(udbase_dir, tokenizer_dir, handparsed_dir, sh
         for treebank in treebanks:
             conllu_file = common.find_treebank_dataset_file(treebank, udbase_dir, dataset, "conllu", fail=True)
             sents.extend(read_sentences_from_conllu(conllu_file))
-        extra_italian = os.path.join(handparsed_dir, "italian-mwt", "italian.mwt")
-        if not os.path.exists(extra_italian):
-            raise FileNotFoundError("Cannot find the extra dataset 'italian.mwt' which includes various multi-words retokenized, expected {}".format(extra_italian))
-        extra_sents = read_sentences_from_conllu(extra_italian)
-        for sentence in extra_sents:
-            if not sentence[2].endswith("_") or not MWT_RE.match(sentence[2]):
-                raise AssertionError("Unexpected format of the italian.mwt file.  Has it already be modified to have SpaceAfter=No everywhere?")
-            sentence[2] = sentence[2][:-1] + "SpaceAfter=No"
-        sents = sents + extra_sents
     else:
         istd_conllu = common.find_treebank_dataset_file("UD_Italian-ISDT", udbase_dir, dataset, "conllu")
         sents = read_sentences_from_conllu(istd_conllu)
@@ -826,12 +762,11 @@ def check_gum_ready(udbase_dir):
     if common.mostly_underscores(gum_conllu):
         raise ValueError("Cannot process UD_English-GUMReddit in its current form.  There should be a download script available in the directory which will help integrate the missing proprietary values.  Please run that script to update the data, then try again.")
 
-def build_combined_english_dataset(udbase_dir, tokenizer_dir, handparsed_dir, short_name, dataset):
+def build_combined_english_dataset(paths, dataset):
     """
-    en_combined is currently EWT, GUM, PUD, and Pronouns
-
-    TODO: use more of the handparsed data
+    en_combined is currently EWT, GUM, PUD, Pronouns, and handparsed
     """
+    udbase_dir = paths["UDBASE"]
     check_gum_ready(udbase_dir)
 
     if dataset == 'train':
@@ -844,10 +779,14 @@ def build_combined_english_dataset(udbase_dir, tokenizer_dir, handparsed_dir, sh
         sents = []
         for treebank in train_treebanks:
             conllu_file = common.find_treebank_dataset_file(treebank, udbase_dir, "train", "conllu", fail=True)
-            sents.extend(read_sentences_from_conllu(conllu_file))
+            new_sents = read_sentences_from_conllu(conllu_file)
+            print("Read %d sentences from %s" % (len(new_sents), conllu_file))
+            sents.extend(new_sents)
         for treebank in test_treebanks:
             conllu_file = common.find_treebank_dataset_file(treebank, udbase_dir, "test", "conllu", fail=True)
-            sents.extend(read_sentences_from_conllu(conllu_file))
+            new_sents = read_sentences_from_conllu(conllu_file)
+            print("Read %d sentences from %s" % (len(new_sents), conllu_file))
+            sents.extend(new_sents)
     else:
         ewt_conllu = common.find_treebank_dataset_file("UD_English-EWT", udbase_dir, dataset, "conllu")
         sents = read_sentences_from_conllu(ewt_conllu)
@@ -855,6 +794,33 @@ def build_combined_english_dataset(udbase_dir, tokenizer_dir, handparsed_dir, sh
     sents = strip_mwt_from_sentences(sents)
     return sents
 
+def build_extra_combined_english_dataset(paths, dataset):
+    """
+    Extra sentences we don't want augmented
+    """
+    handparsed_dir = paths["HANDPARSED_DIR"]
+    sents = []
+    if dataset == 'train':
+        sents.extend(read_sentences_from_conllu(os.path.join(handparsed_dir, "english-handparsed", "english.conll")))
+    return sents
+
+def build_extra_combined_italian_dataset(paths, dataset):
+    """
+    Extra data - the MWT data for Italian
+    """
+    handparsed_dir = paths["HANDPARSED_DIR"]
+    if dataset != 'train':
+        return []
+
+    extra_italian = os.path.join(handparsed_dir, "italian-mwt", "italian.mwt")
+    if not os.path.exists(extra_italian):
+        raise FileNotFoundError("Cannot find the extra dataset 'italian.mwt' which includes various multi-words retokenized, expected {}".format(extra_italian))
+    extra_sents = read_sentences_from_conllu(extra_italian)
+    for sentence in extra_sents:
+        if not sentence[2].endswith("_") or not MWT_RE.match(sentence[2]):
+            raise AssertionError("Unexpected format of the italian.mwt file.  Has it already be modified to have SpaceAfter=No everywhere?")
+        sentence[2] = sentence[2][:-1] + "SpaceAfter=No"
+    return extra_sents
 
 def replace_semicolons(sentences):
     """
@@ -884,13 +850,16 @@ def replace_semicolons(sentences):
     print("Updated %d sentences to replace sentence-final ; with ." % count)
     return new_sents
 
-def build_combined_spanish_dataset(udbase_dir, tokenizer_dir, handparsed_dir, short_name, dataset):
+def build_combined_spanish_dataset(paths, dataset):
     """
     es_combined is AnCora and GSD put together
 
     TODO: remove features which aren't shared between datasets
     TODO: consider mixing in PUD?
     """
+    udbase_dir = paths["UDBASE"]
+    tokenizer_dir = paths["TOKENIZE_DATA_DIR"]
+    handparsed_dir = paths["HANDPARSED_DIR"]
     if dataset == 'train':
         treebanks = ["UD_Spanish-AnCora", "UD_Spanish-GSD"]
         sents = []
@@ -913,20 +882,91 @@ def build_combined_spanish_dataset(udbase_dir, tokenizer_dir, handparsed_dir, sh
     return sents
 
 
+def build_combined_hebrew_dataset(paths, dataset):
+    """
+    Combines the IAHLT treebank with an updated form of HTB where the annotation style more closes matches IAHLT
+
+    Currently the updated HTB is not in UD, so you will need to clone
+    git@github.com:IAHLT/UD_Hebrew.git to $UDBASE_GIT
+
+    dev and test sets will be those from IAHLT
+    """
+    udbase_dir = paths["UDBASE"]
+    udbase_git_dir = paths["UDBASE_GIT"]
+
+    treebanks = ["UD_Hebrew-IAHLTwiki"]
+    if dataset == 'train':
+        sents = []
+        for treebank in treebanks:
+            conllu_file = common.find_treebank_dataset_file(treebank, udbase_dir, dataset, "conllu", fail=True)
+            new_sents = read_sentences_from_conllu(conllu_file)
+            print("Read %d sentences from %s" % (len(new_sents), conllu_file))
+            sents.extend(new_sents)
+
+        # if/when this gets ported back to UD, switch to getting both datasets from UD
+        hebrew_git_dir = os.path.join(udbase_git_dir, "UD_Hebrew")
+        if not os.path.exists(hebrew_git_dir):
+            raise FileNotFoundError("Please download git@github.com:IAHLT/UD_Hebrew.git to %s (based on $UDBASE_GIT)" % hebrew_git_dir)
+        conllu_file = os.path.join(hebrew_git_dir, "he_htb-ud-train.conllu")
+        if not os.path.exists(conllu_file):
+            raise FileNotFoundError("Found %s but inexplicably there was no %s" % (hebrew_git_dir, conllu_file))
+        new_sents = read_sentences_from_conllu(conllu_file)
+        print("Read %d sentences from %s" % (len(new_sents), conllu_file))
+        sents.extend(new_sents)
+    else:
+        conllu_file = common.find_treebank_dataset_file(treebanks[0], udbase_dir, dataset, "conllu", fail=True)
+        sents = read_sentences_from_conllu(conllu_file)
+
+    return sents
+
 COMBINED_FNS = {
     "en_combined": build_combined_english_dataset,
     "es_combined": build_combined_spanish_dataset,
+    "he_combined": build_combined_hebrew_dataset,
     "it_combined": build_combined_italian_dataset,
 }
 
-def build_combined_dataset(udbase_dir, tokenizer_dir, handparsed_dir, short_name, augment):
+# some extra data for the combined models without augmenting
+COMBINED_EXTRA_FNS = {
+    "en_combined": build_extra_combined_english_dataset,
+    "it_combined": build_extra_combined_italian_dataset,
+}
+
+def build_combined_dataset(paths, short_name, augment):
     random.seed(1234)
+    tokenizer_dir = paths["TOKENIZE_DATA_DIR"]
     build_fn = COMBINED_FNS[short_name]
+    extra_fn = COMBINED_EXTRA_FNS.get(short_name, None)
     for dataset in ("train", "dev", "test"):
-        output_conllu = f"{tokenizer_dir}/{short_name}.{dataset}.gold.conllu"
-        sents = build_fn(udbase_dir, tokenizer_dir, handparsed_dir, short_name, dataset)
+        output_conllu = common.tokenizer_conllu_name(tokenizer_dir, short_name, dataset)
+        sents = build_fn(paths, dataset)
         if dataset == 'train' and augment:
             sents = augment_punct(sents)
+        if extra_fn is not None:
+            sents.extend(extra_fn(paths, dataset))
+        write_sentences_to_conllu(output_conllu, sents)
+
+BIO_DATASETS = ("en_craft", "en_genia", "en_mimic")
+
+def build_bio_dataset(paths, udbase_dir, tokenizer_dir, handparsed_dir, short_name, augment):
+    """
+    Process the en bio datasets
+
+    Creates a dataset by combining the en_combined data with one of the bio sets
+    """
+    random.seed(1234)
+    name, bio_dataset = short_name.split("_")
+    assert name == 'en'
+    for dataset in ("train", "dev", "test"):
+        output_conllu = common.tokenizer_conllu_name(tokenizer_dir, short_name, dataset)
+        if dataset == 'train':
+            sents = build_combined_english_dataset(paths, dataset)
+            if dataset == 'train' and augment:
+                sents = augment_punct(sents)
+        else:
+            sents = []
+        bio_file = os.path.join(paths["BIO_UD_DIR"], "UD_English-%s" % bio_dataset.upper(), "en_%s-ud-%s.conllu" % (bio_dataset.lower(), dataset))
+        sents.extend(read_sentences_from_conllu(bio_file))
         write_sentences_to_conllu(output_conllu, sents)
 
 def build_combined_english_gum_dataset(udbase_dir, tokenizer_dir, short_name, dataset, augment):
@@ -938,7 +978,7 @@ def build_combined_english_gum_dataset(udbase_dir, tokenizer_dir, short_name, da
     check_gum_ready(udbase_dir)
     random.seed(1234)
 
-    output_conllu = f"{tokenizer_dir}/{short_name}.{dataset}.gold.conllu"
+    output_conllu = common.tokenizer_conllu_name(tokenizer_dir, short_name, dataset)
 
     treebanks = ["UD_English-GUM", "UD_English-GUMReddit"]
     sents = []
@@ -955,9 +995,12 @@ def build_combined_english_gum(udbase_dir, tokenizer_dir, short_name, augment):
     for dataset in ("train", "dev", "test"):
         build_combined_english_gum_dataset(udbase_dir, tokenizer_dir, short_name, dataset, augment)
 
-def prepare_ud_dataset(treebank, udbase_dir, tokenizer_dir, short_name, short_language, dataset, augment=True):
-    input_conllu = common.find_treebank_dataset_file(treebank, udbase_dir, dataset, "conllu", fail=True)
-    output_conllu = f"{tokenizer_dir}/{short_name}.{dataset}.gold.conllu"
+def prepare_ud_dataset(treebank, udbase_dir, tokenizer_dir, short_name, short_language, dataset, augment=True, input_conllu=None, output_conllu=None):
+    if input_conllu is None:
+        input_conllu = common.find_treebank_dataset_file(treebank, udbase_dir, dataset, "conllu", fail=True)
+    if output_conllu is None:
+        output_conllu = common.tokenizer_conllu_name(tokenizer_dir, short_name, dataset)
+    print("Reading from %s and writing to %s" % (input_conllu, output_conllu))
 
     if short_name == "te_mtg" and dataset == 'train' and augment:
         write_augmented_dataset(input_conllu, output_conllu, augment_telugu)
@@ -968,7 +1011,8 @@ def prepare_ud_dataset(treebank, udbase_dir, tokenizer_dir, short_name, short_la
     elif dataset == 'train' and augment:
         write_augmented_dataset(input_conllu, output_conllu, augment_punct)
     else:
-        shutil.copyfile(input_conllu, output_conllu)
+        sents = read_sentences_from_conllu(input_conllu)
+        write_sentences_to_conllu(output_conllu, sents)
 
 def process_ud_treebank(treebank, udbase_dir, tokenizer_dir, short_name, short_language, augment=True):
     """
@@ -1003,8 +1047,15 @@ def process_partial_ud_treebank(treebank, udbase_dir, tokenizer_dir, short_name,
       UD_Welsh-CCG
     """
     train_input_conllu = common.find_treebank_dataset_file(treebank, udbase_dir, "train", "conllu")
-    train_output_conllu = f"{tokenizer_dir}/{short_name}.train.gold.conllu"
-    dev_output_conllu = f"{tokenizer_dir}/{short_name}.dev.gold.conllu"
+    test_input_conllu = common.find_treebank_dataset_file(treebank, udbase_dir, "test", "conllu")
+
+    train_output_conllu = common.tokenizer_conllu_name(tokenizer_dir, short_name, "train")
+    dev_output_conllu = common.tokenizer_conllu_name(tokenizer_dir, short_name, "dev")
+    test_output_conllu = common.tokenizer_conllu_name(tokenizer_dir, short_name, "test")
+
+    if (common.num_words_in_file(train_input_conllu) <= 1000 and
+        common.num_words_in_file(test_input_conllu) > 5000):
+        train_input_conllu, test_input_conllu = test_input_conllu, train_input_conllu
 
     if not split_train_file(treebank=treebank,
                             train_input_conllu=train_input_conllu,
@@ -1014,7 +1065,7 @@ def process_partial_ud_treebank(treebank, udbase_dir, tokenizer_dir, short_name,
 
     # the test set is already fine
     # currently we do not do any augmentation of these partial treebanks
-    prepare_ud_dataset(treebank, udbase_dir, tokenizer_dir, short_name, short_language, "test", augment=False)
+    prepare_ud_dataset(treebank, udbase_dir, tokenizer_dir, short_name, short_language, "test", augment=False, input_conllu=test_input_conllu, output_conllu=test_output_conllu)
 
 def add_specific_args(parser):
     parser.add_argument('--no_augment', action='store_false', dest='augment', default=True,
@@ -1024,6 +1075,7 @@ def add_specific_args(parser):
     convert_th_lst20.add_lst20_args(parser)
 
     convert_vi_vlsp.add_vlsp_args(parser)
+
 def process_treebank(treebank, paths, args):
     """
     Processes a single treebank into train, dev, test parts
@@ -1038,12 +1090,14 @@ def process_treebank(treebank, paths, args):
     tokenizer_dir = paths["TOKENIZE_DATA_DIR"]
     handparsed_dir = paths["HANDPARSED_DIR"]
 
-    short_name = common.project_to_short_name(treebank)
+    short_name = treebank_to_short_name(treebank)
     short_language = short_name.split("_")[0]
 
     os.makedirs(tokenizer_dir, exist_ok=True)
 
-    if short_name == "vi_vlsp":
+    if short_name == "my_alt":
+        convert_my_alt.convert_my_alt(paths["CONSTITUENCY_BASE"], tokenizer_dir)
+    elif short_name == "vi_vlsp":
         convert_vi_vlsp.convert_vi_vlsp(paths["EXTERN_DIR"], tokenizer_dir, args)
     elif short_name == "th_orchid":
         convert_th_orchid.main(paths["EXTERN_DIR"], tokenizer_dir)
@@ -1053,8 +1107,10 @@ def process_treebank(treebank, paths, args):
         convert_th_best.main(paths["EXTERN_DIR"], tokenizer_dir)
     elif short_name.startswith("ko_combined"):
         build_combined_korean(udbase_dir, tokenizer_dir, short_name)
-    elif short_name in ("it_combined", "en_combined", "es_combined"):
-        build_combined_dataset(udbase_dir, tokenizer_dir, handparsed_dir, short_name, args.augment)
+    elif short_name in COMBINED_FNS: # eg "it_combined", "en_combined", etc
+        build_combined_dataset(paths, short_name, args.augment)
+    elif short_name in BIO_DATASETS:
+        build_bio_dataset(paths, udbase_dir, tokenizer_dir, handparsed_dir, short_name, args.augment)
     elif short_name.startswith("en_gum"):
         # we special case GUM because it should include a filled-out GUMReddit
         print("Preparing data for %s: %s, %s" % (treebank, short_name, short_language))
@@ -1071,10 +1127,10 @@ def process_treebank(treebank, paths, args):
             process_ud_treebank(treebank, udbase_dir, tokenizer_dir, short_name, short_language, args.augment)
 
     if not short_name in ('th_orchid', 'th_lst20'):
-        convert_conllu_to_txt(tokenizer_dir, short_name)
+        common.convert_conllu_to_txt(tokenizer_dir, short_name)
 
     if args.prepare_labels:
-        prepare_treebank_labels(tokenizer_dir, short_name)
+        common.prepare_tokenizer_treebank_labels(tokenizer_dir, short_name)
 
 
 def main():
